/*
 *  linux/kernel/mips/sys_call.S
 *
 *  Copyright (C) 1994 Waldorf Electronics
 *  written by Ralf Baechle
 */

/*
 * sys_call.S  contains the system-call and fault low-level handling routines.
 * This also contains the timer-interrupt handler, as well as all interrupts
 * and faults that can result in a task-switch.
 */

#define __ASSEMBLY__

#include <linux/sys.h>
#include <asm/segment.h>
#include <asm/mipsregs.h>
#include <asm/mipsconfig.h>
#include <asm/mm.h>
#include <asm/stackframe.h>
#include <asm/regdef.h>

/*
 * These are offsets into the task-struct.
 */
state		=  0
counter		=  4
priority	=  8
signal		= 12
blocked		= 16
flags		= 20
errno		= 24			#/* MIPS OK */
exec_domain	= 60			#/* ??? */

ENOSYS		= 38

#define SIGSEGV 11
#define SIGFPE  8
#define SIGILL  4

		.globl	_system_call
		.globl	ret_from_sys_call
		.globl	_sys_call_table

		.text
		.set	noreorder
		.align 4
handle_bottom_half:
		la	s0,_intr_count
		lw	s1,(s0)
		mfc0	s3,CP0_STATUS			# Enable IRQs
		addiu	s2,s0,1
		sw	s2,(s0)
		ori	t0,s3,7
		xori	t0,t0,6
		jal	_do_bottom_half
		mtc0	t0,CP0_STATUS
		mtc0	s3,CP0_STATUS			# Restore old IRQ state
		j	9f
		sw	s1,(s0)				# delay slot
		.set	reorder
		.align	4
reschedule:
		la	ra,ret_from_sys_call
		j	_schedule
		nop

		.align	4
_handle_sys:
_system_call:
		.set	noreorder
		.set	noat
		SAVE_ALL
		.set	at
		ENABLE_IRQ
#if 1
	la	a0,sys_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
#endif
		/*
		 * Compute return address. We assume that syscalls never
		 * appear in branch delay slots.
		 */
		lw	t3,FR_EPC(sp)
		lw	s1,FR_REG1(sp)
		li	t0,-ENOSYS
		addiu	t3,t3,4
		sw	t3,FR_EPC(sp)
		li	t2,NR_syscalls
		bge	s1,t2,ret_from_sys_call
		sw	t0,FR_ORIG_REG2(sp)	# delay slot
		sll	s1,s1,2
		lw	s1,_sys_call_table(s1)
		lw	s0,_current

		beq	zero,s1,ret_from_sys_call
		lw	t0,flags(s0)
		sll	t0,t0,2			# PF_TRACESYS
		bltz	t0,1f
		sw	zero,errno(s0)		# delay slot

		lw	a0,FR_REG2(sp)
		lw	a1,FR_REG3(sp)
		lw	a2,FR_REG4(sp)
		lw	a3,FR_REG5(sp)
		lw	t0,FR_REG6(sp)
		jalr	s1			# do the real work
		sw	t0,16(sp)		# delay slot

		lw	t0,errno(s0)
		sw	v0,FR_REG2(sp)		# save the return value
		subu	t0,zero,t0		# t0 = -t0
		beq	zero,t0,ret_from_sys_call
		nop
		/*
		 * Fixme: should set error flag
		 */
		j	ret_from_sys_call
		sw	t0,FR_REG2(sp)		# delay slot

		.align	4
1:		jal	_syscall_trace
		nop				# delay slot

		lw	a0,FR_REG2(sp)
		lw	a1,FR_REG3(sp)
		lw	a2,FR_REG4(sp)
		lw	a3,FR_REG5(sp)
		lw	t0,FR_REG6(sp)
		jalr	s1			# do the real work
		sw	t0,16(sp)		# delay slot

		lw	t0,errno(s0)
		sw	v0,FR_REG2(sp)		# save the return value
		subu	t0,zero,t0
		beq	zero,t0,1f
		nop				# delay slot
		sw	t1,FR_REG2(sp)
		/*
		 * Fixme: should set error flag
		 */
1:		jal	_syscall_trace
		nop

		.align	4
ret_from_sys_call:
		lw	t0,_intr_count		# bottom half
		bne	zero,t0,2f
9:
		lw	t0,_bh_mask		# delay slot
		lw	t1,_bh_active		# unused delay slot
		and	t0,t0,t1
		bne	zero,t0,handle_bottom_half

		lw	t0,FR_STATUS(sp)	# returning to supervisor ?
		andi	t1,t0,0x10
		beq	zero,t1,2f

		mfc0	t0,CP0_STATUS		# delay slot
		ori	t0,t0,0x7		# enable irqs
		lw	t1,_need_resched
		xori	t0,t0,0x6
		bne	zero,t1,reschedule
		mtc0	t0,CP0_STATUS		# delay slot

		lw	s0,_current
		lw	t0,_task
		lw	t1,state(s0)		# state
		beq	s0,t0,2f		# task[0] cannot have signals
		lw	t0,counter(s0)		# counter
		bne	zero,t1,reschedule	# state == 0 ?
		lw	a0,blocked(s0)
						# save blocked in a0 for
						# signal handling
		beq	zero,t0,reschedule	# counter == 0 ?
		lw	t0,signal(s0)
		nor	t1,zero,a0
		and	t1,t0,t1
		beq	zero,t1,skip_signal_return
		nop

		jal	_do_signal
		addiu	a1,sp,20		# delay slot
		
skip_signal_return:
		.set	noreorder
		.set	noat
2:
return:		RESTORE_ALL
		.set	at

/*
 * Assumptions for _handle_int:
 *  - only bank a or b are possible interrupt sources
 */
		.text
		.globl	_handle_int
_handle_int:
		.set	noreorder
		.set	noat
#if 0
	la	a0,int_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
#endif
		SAVE_ALL
		ENABLE_IRQ
		.set	at
		la	s0,PORT_BASE
		li	t1,0x0f
		sb	t1,0x20(s0)		# poll command
		lb	t1,0x20(s0)		# read result
		FILL_LDS
		bgtz	t1,poll_second
		andi	t1,t1,7
		/*
		 * Acknowledge first pic
		 */
		lb	t2,0x21(s0)
		li	s1,1
		sllv	s1,s1,t1
		lb	t4,_cache_21
		or	t4,t4,s1
		sb	t4,_cache_21
		sb	t4,0x21(s0)
		li	t4,0x20
		sb	t4,0x20(s0)
		lw	t0,_intr_count
		addiu	t0,t0,1
		sw	t0,_intr_count
		/*
		 * Now call the real handler
		 */
		la	t0,_IRQ_vectors
		sll	t2,t1,2
		addu	t0,t0,t2
		lw	t0,(t0)
		FILL_LDS
		jalr	t0
		nop
		lw	t0,_intr_count
		subu	t0,t0,1
		sw	t0,_intr_count
		/*
		 * Unblock first pic
		 */
		lbu	t1,0x21(s0)
		lb	t1,_cache_21
		nor	s1,zero,s1
		and	t1,t1,s1
		sb	t1,_cache_21
		jr	v0
		sb	t1,0x21(s0)		# delay slot

poll_second:
		li	t1,0x0f
		sb	t1,0xa0(s0)		# poll command
		lb	t1,0xa0(s0)		# read result
		FILL_LDS
		bgtz	t1,spurious_interrupt
		andi	t1,t1,7
		/*
		 * Acknowledge second pic
		 */
		lbu	t2,0xa1(s0)
		lbu	t3,_cache_A1
		li	s1,1
		sllv	s1,s1,t1
		or	t3,t3,s1
		sb	t3,_cache_A1
		sb	t3,0xa1(s0)
		li	t3,0x20
		sb	t3,0xa0(s0)
		lw	t0,_intr_count
		sb	t3,0x20(s0)
		addiu	t0,t0,1
		sw	t0,_intr_count
		/*
		 * Now call the real handler
		 */
		la	t0,_IRQ_vectors
		sll	t2,t1,2
		addu	t0,t0,t2
		lw	t0,32(t0)
		FILL_LDS
		jalr	t0
		nop					# delay slot
		lw	t0,_intr_count
		subu	t0,t0,1
		sw	t0,_intr_count
		/*
		 * Unblock second pic
		 */
		lbu	t1,0xa1(s0)
		lb	t1,_cache_A1
		nor	s1,zero,s1
		and	t1,t1,s1
		sb	t1,_cache_A1
		jr	v0
		sb	t1,0xa1(s0)			# delay slot

spurious_interrupt:
		/*
		 * Nothing happend... (whistle)
		 */
#if 0
	la	a0,spurious_text
	jal	_printk
	nop
#endif
		lw	t0,_spurious_count
		la	v0,return
		addiu	t0,t0,1
		sw	t0,_spurious_count
		jr	ra
		nop
		
		.globl	_IRQ
_IRQ:		move	s2,ra
		mfc0	t0,CP0_STATUS
		ori	t0,t0,0x1f
		xori	t0,t0,0x1e
		mtc0	t0,CP0_STATUS

#if 0
	move	s7,t1
	la	a0,hell_text
	move	a1,s7
	addiu	a2,sp,20
	jal	_printk
	nop
	move	t1,s7
#endif

		move	a0,t1
		jal	_do_IRQ
		addiu	a1,sp,20			# delay slot

#if 0
	la	a0,hello_text
	jal	_printk
	nop
#endif

		mfc0	t0,CP0_STATUS
		ori	t0,t0,1
		xori	t0,t0,1
		la	v0,ret_from_sys_call
		jr	s2
		mtc0	t0,CP0_STATUS			# delay slot

		.globl	_fast_IRQ
_fast_IRQ:	move	s2,ra
		move	a0,t1				# delay slot
		jal	_do_fast_IRQ
		addiu	a1,sp,20
		la	v0,return
		jr	s2
		nop					# delay slot

		.globl	_bad_IRQ
_bad_IRQ:
		/*
		 * Don't return & unblock the pic
		 */
		j	return
		nop

		.bss
		.globl	_IRQ_vectors

_IRQ_vectors:
		.fill	16,4,0

/*
 * Dummy handlers
 */
		.text
		.globl	_handle_tlbl
_handle_tlbl:
		.set	noreorder
		.set	noat
		/*
		 * Check whether this is a refill or an invalid exception
		 *
		 * NOTE: Some MIPS manuals say that the R4x00 sets the
		 * BadVAddr only when EXL == 0. This is wrong - BadVaddr
		 * is being set for all Reload, Invalid and Modified
		 * exceptions.
		 */
		dmfc0	k0,CP0_BADVADDR
		dmfc0	k1,CP0_ENTRYHI
		ori	k0,k0,0x1fff
		xori	k0,k0,0x1fff
		andi	k1,k1,0xff
		or	k0,k0,k1
		dmfc0	k1,CP0_ENTRYHI
		dmtc0	k0,CP0_ENTRYHI
		tlbp
		mfc0	k0,CP0_INDEX
		srl	k0,k0,31
		beq	zero,k0,invalid_tlbl
		dmtc0	k1,CP0_ENTRYHI			# delay slot
		/*
		 * Not in tlb -> nested refill exception
		 * Load the missing entry and return. This is the most
		 * efficient way to regain the faulting address.
		 */
		dmfc0	k1,CP0_CONTEXT
		dsra	k1,k1,1
		lwu	k0,(k1)			# Never causes another exception
#if 0
	la	a0,tlbl2_text
	move	a1,k1
	jal	_printk
	nop
1:	j	1b
	nop
#endif
		lwu	k1,4(k1)
		dsrl	k0,k0,6			# Convert to EntryLo format
		dsrl	k1,k1,6			# Convert to EntryLo format
		dmtc0	k0,CP0_ENTRYLO0
		dmtc0	k1,CP0_ENTRYLO1
		nop;nop;nop;nop;nop;nop;nop;nop
		tlbwr
		nop;nop;nop;nop;nop;nop;nop;nop
		/*
		 * No need to handle branch delay slots.
		 */
		eret

		/*
		 * Handle invalid exception
		 *
		 * There are two possible causes for an invalid (tlbl)
		 * exception:
		 * 1) pages that have the present bit set but the valid bit
		 *    unset.
		 * 2) pages that don't exist
		 * Case one needs fast handling, therefore don't save
		 * registers yet.
		 *
		 * k0 now contains the bad virtual address.
		 */
invalid_tlbl:
#if 1
	la	a0,tlbl3_text
	mfc0	a1,CP0_BADVADDR
	dmfc0	a2,CP0_CONTEXT
	dsra	a2,a2,1
	jal	_printk
	nop
#	jal	_dump_tlb
#	nop
1:	j	1b
	nop
#endif
		/*
		 * Remove entry so we don't need to care later
		 */
		mfc0	k0,CP0_INDEX
		lui	k1,0x0008
		or	k0,k0,k1
		dsll	k0,k0,13
		dmtc0	k0,CP0_ENTRYHI
		dmtc0	zero,CP0_ENTRYLO0
		dmtc0	zero,CP0_ENTRYLO1
		tlbwi
		/*
		 * Test whether present bit in entry is set
		 */
		dmfc0	k0,CP0_BADVADDR
		srl	k0,k0,10
		lui	k1,TLBMAP_HIGH
		addu	k0,k0,k1
		ori	k0,k0,3
		xori	k0,k0,3
		lw	k1,(k0)
		andi	k1,k1,PAGE_PRESENT
		beq	zero,k1,nopage_tlbl
		/*
		 * Present bit is set -> set valid and accessed bits
		 */
		lw	k1,(k0)				# delay slot
		ori	k1,k1,PAGE_ACCESSED
		sw	k1,(k0)
		/*
		 * No need to handle branch delay slots.
		 */
		eret

		/*
		 * Page doesn't exist. Lots of work which is less important
		 * for speed needs to be done, so hand it all over to the
		 * kernel memory management routines.
		 */
nopage_tlbl:
		SAVE_ALL
		.set	at
		ENABLE_IRQ
		/*
		 * Create a Intel-style errorcode
		 * Bit 0: P Present
		 *        0 == Page not in memory
		 *        1 == privilege violation
		 * Bit 1: R/W Read/Write
		 *        0 == ReadAccess
		 *        1 == WriteAccess
		 * Bit 2: U/S User/Supervisor
		 *        0 == User mode
		 *        1 == Kernel mode
		 *
		 * a0 (struct pt_regs *) regs
		 * a1 (unsigned long)    error_code
		 */
		lw	a1,FR_STATUS(sp)
		addiu	a0,sp,20
		srl	a1,a1,4
		andi	a1,a1,1
		jal	_do_page_fault
		xori	a1,a1,1				# delay slot
		j	ret_from_sys_call
		nop					# delay slot

		.text
		.globl	_handle_tlbs
_handle_tlbs:
		.set	noreorder
		.set	noat
#if 1
	la	a0,tlbs_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
#endif
		/*
		 * It is impossible that is a nested reload exception.
		 * Therefore this must be a invalid exception.
		 * Two possible cases:
		 * 1) Page not used yet
		 * 2) Page doesn't exist yet. Let the kernel handle the trouble.
		 *
		 * Test whether present bit in entry is set
		 */
		dmfc0	k0,CP0_BADVADDR
		srl	k0,k0,10
		lui	k1,TLBMAP_HIGH
		addu	k0,k0,k1
		ori	k0,k0,3
		xori	k0,k0,3
		lw	k1,(k0)
		andi	k1,k1,(PAGE_PRESENT|PAGE_RW)
		beq	zero,k1,nopage_tlbs
		/*
		 * Present and writable bits set -> set accessed and dirty bits.
		 */
		lw	k1,(k0)				# delay slot
		ori	k1,k1,PAGE_ACCESSED|PAGE_DIRTY
		sw	k1,(k0)
		/*
		 * Now reload the entry into the tlb
		 */
		ori	k0,k0,0x1000
		xori	k0,k0,0x1000
		lw	k1,4(k0)
		lw	k0,(k0)
		srl	k0,k0,6
		srl	k1,k1,6
		dmtc0	k0,CP0_ENTRYLO0
		dmtc0	k1,CP0_ENTRYLO1
		tlbwi
		/*
		 * No need to handle branch delay slots.
		 */
		eret

		/*
		 * Page doesn't exist. Lots of work which is less important
		 * for speed needs to be done, so hand it all over to the
		 * kernel memory management routines.
		 */
nopage_tlbs:
nowrite_mod:
		/*
		 * Remove entry so we don't need to care later
		 */
		mfc0	k0,CP0_INDEX
		lui	k1,0x0008
		or	k0,k0,k1
		dsll	k0,k0,13
		dmtc0	k0,CP0_ENTRYHI
		dmtc0	zero,CP0_ENTRYLO0
		dmtc0	zero,CP0_ENTRYLO1
		tlbwi
		SAVE_ALL
		.set	at
		ENABLE_IRQ
		/*
		 * Create a Intel-style errorcode
		 * Bit 0: P Present
		 *        0 == Page not in memory
		 *        1 == privilege violation
		 * Bit 1: R/W Read/Write
		 *        0 == ReadAccess
		 *        1 == WriteAccess
		 * Bit 2: U/S User/Supervisor
		 *        0 == User mode
		 *        1 == Kernel mode
		 *
		 * a0 (struct pt_regs *) regs
		 * a1 (unsigned long)    error_code
		 */
		lw	a1,FR_STATUS(sp)
		addiu	a0,sp,20
		srl	a1,a1,4
		andi	a1,a1,1
		jal	_do_page_fault
		xori	a1,a1,3				# branch delay slot
		j	ret_from_sys_call
		nop					# branch delay slot

		.globl	_handle_mod
_handle_mod:
		.set	noreorder
		.set	noat
#if 1
	la	a0,mod_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
#endif
		/*
		 * Two possible cases:
		 * 1) Page is rw but not dirty -> set dirty and return
		 * 2) Page is not rw -> call C handler
		 */
		dmfc0	k0,CP0_BADVADDR
		srl	k0,k0,10
		lui	k1,TLBMAP_HIGH
		addu	k0,k0,k1
		ori	k0,k0,3
		xori	k0,k0,3
		lw	k1,(k0)
		andi	k1,k1,PAGE_RW
		beq	zero,k1,nopage_tlbs
		/*
		 * Present and writable bits set -> set accessed and dirty bits.
		 */
		lw	k1,(k0)				# delay slot
		ori	k1,k1,(PAGE_ACCESSED|PAGE_DIRTY)
		sw	k1,(k0)
		/*
		 * Now reload the entry into the tlb
		 */
		ori	k0,k0,0x1000
		xori	k0,k0,0x1000
		lw	k1,4(k0)
		lw	k0,(k0)
		srl	k0,k0,6
		srl	k1,k1,6
		dmtc0	k0,CP0_ENTRYLO0
		dmtc0	k1,CP0_ENTRYLO1
		tlbwi
		/*
		 * No need to handle branch delay slots.
		 */
		eret

		.globl	_handle_adel
_handle_adel:
		.globl	_handle_ades
_handle_ades:
		.globl	_handle_ibe
_handle_ibe:
		.globl	_handle_dbe
_handle_dbe:
		.set	noreorder
		.set	noat
#if 1
	la	a0,ab_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
freeze_ab: j freeze_ab
	nop
#endif
		SAVE_ALL
		ENABLE_IRQ
		li	a0,SIGSEGV
		la	a1,_current
		jal	_send_sig
		li	a2,1				# branch delay slot
		/*
		 * No need to handle branch delay slots.
		 */
		j	ret_from_sys_call
		nop					# jump delay slot

		.globl	_handle_ov
_handle_ov:
		.globl	_handle_fpe
_handle_fpe:
		.set	noreorder
		.set	noat
		SAVE_ALL
		ENABLE_IRQ
#if 1
	la	a0,ovfp_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
#endif
		li	a0,SIGFPE
		la	a1,_current
		jal	_send_sig
		li	a2,1				# branch delay slot
		/*
		 * No need to handle branch delay slots.
		 */
		j	ret_from_sys_call
		nop					# branch delay slot

		.globl	_handle_bp
_handle_bp:
		.globl	_handle_tr
_handle_tr:
		.globl	_handle_ri
_handle_ri:
		.globl	_handle_cpu
_handle_cpu:
		.set	noreorder
		.set	noat
		SAVE_ALL
		ENABLE_IRQ
#if 1
	la	a0,cpu_text
	mfc0	a1,CP0_BADVADDR
	jal	_printk
	nop
#endif
		li	a0,SIGILL
		la	a1,_current
		jal	_send_sig
		li	a2,1				# branch delay slot
		j	ret_from_sys_call
		nop					# branch delay slot

		/*
		 * Famous last words: not used...
		 */
		.globl	_handle_reserved
_handle_reserved:
		.set	noreorder
		.set	noat
		SAVE_ALL
		la	a0,reserved_text
		j	_panic
		nop

		.data
reserved_text:	.asciz	"reserved exception"

/*
 * Exception handler table, 32 entries.
 */
		.globl	_exception_handlers
_exception_handlers:
		.word	_handle_int		/*  0 */
		.word	_handle_mod		/*  1 */
		.word	_handle_tlbl		/*  2 */
		.word	_handle_tlbs		/*  3 */
		.word	_handle_adel		/*  4 */
		.word	_handle_ades		/*  5 */
		.word	_handle_ibe		/*  6 */
		.word	_handle_dbe		/*  7 */
		.word	_handle_sys		/*  8 */
		.word	_handle_bp		/*  9 */
		.word	_handle_ri		/* 10 */
		.word	_handle_cpu		/* 11 */
		.word	_handle_ov		/* 12 */
		.word	_handle_tr		/* 13 */
		.word	_handle_reserved	/* 14 */
		.word	_handle_fpe             /* 15 */
		.word	_handle_reserved	/* 16 */
		.word	_handle_reserved	/* 17 */
		.word	_handle_reserved	/* 18 */
		.word	_handle_reserved	/* 19 */
		.word	_handle_reserved	/* 20 */
		.word	_handle_reserved	/* 21 */
		.word	_handle_reserved	/* 22 */
		.word	_handle_reserved	/* 23 */
		.word	_handle_reserved	/* 24 */
		.word	_handle_reserved	/* 25 */
		.word	_handle_reserved	/* 26 */
		.word	_handle_reserved	/* 27 */
		.word	_handle_reserved	/* 28 */
		.word	_handle_reserved	/* 29 */
		.word	_handle_reserved	/* 30 */
		.word	_handle_reserved	/* 31 */

/*
 * Table of syscalls
 */
		.data
_sys_call_table:
		.word	_sys_setup		/* 0 */
		.word	_sys_exit
		.word	_sys_fork
		.word	_sys_read
		.word	_sys_write
		.word	_sys_open		/* 5 */
		.word	_sys_close
		.word	_sys_waitpid
		.word	_sys_creat
		.word	_sys_link
		.word	_sys_unlink		/* 10 */
		.word	_sys_execve
		.word	_sys_chdir
		.word	_sys_time
		.word	_sys_mknod
		.word	_sys_chmod		/* 15 */
		.word	_sys_chown
		.word	_sys_break
		.word	_sys_stat
		.word	_sys_lseek
		.word	_sys_getpid		/* 20 */
		.word	_sys_mount
		.word	_sys_umount
		.word	_sys_setuid
		.word	_sys_getuid
		.word	_sys_stime		/* 25 */
		.word	_sys_ptrace
		.word	_sys_alarm
		.word	_sys_fstat
		.word	_sys_pause
		.word	_sys_utime		/* 30 */
		.word	_sys_stty
		.word	_sys_gtty
		.word	_sys_access
		.word	_sys_nice
		.word	_sys_ftime		/* 35 */
		.word	_sys_sync
		.word	_sys_kill
		.word	_sys_rename
		.word	_sys_mkdir
		.word	_sys_rmdir		/* 40 */
		.word	_sys_dup
		.word	_sys_pipe
		.word	_sys_times
		.word	_sys_prof
		.word	_sys_brk		/* 45 */
		.word	_sys_setgid
		.word	_sys_getgid
		.word	_sys_signal
		.word	_sys_geteuid
		.word	_sys_getegid		/* 50 */
		.word	_sys_acct
		.word	_sys_phys
		.word	_sys_lock
		.word	_sys_ioctl
		.word	_sys_fcntl		/* 55 */
		.word	_sys_mpx
		.word	_sys_setpgid
		.word	_sys_ulimit
		.word	_sys_olduname
		.word	_sys_umask		/* 60 */
		.word	_sys_chroot
		.word	_sys_ustat
		.word	_sys_dup2
		.word	_sys_getppid
		.word	_sys_getpgrp		/* 65 */
		.word	_sys_setsid
		.word	_sys_sigaction
		.word	_sys_sgetmask
		.word	_sys_ssetmask
		.word	_sys_setreuid		/* 70 */
		.word	_sys_setregid
		.word	_sys_sigsuspend
		.word	_sys_sigpending
		.word	_sys_sethostname
		.word	_sys_setrlimit		/* 75 */
		.word	_sys_getrlimit
		.word	_sys_getrusage
		.word	_sys_gettimeofday
		.word	_sys_settimeofday
		.word	_sys_getgroups		/* 80 */
		.word	_sys_setgroups
		.word	_sys_select
		.word	_sys_symlink
		.word	_sys_lstat
		.word	_sys_readlink		/* 85 */
		.word	_sys_uselib
		.word	_sys_swapon
		.word	_sys_reboot
		.word	_sys_readdir
		.word	_sys_mmap		/* 90 */
		.word	_sys_munmap
		.word	_sys_truncate
		.word	_sys_ftruncate
		.word	_sys_fchmod
		.word	_sys_fchown		/* 95 */
		.word	_sys_getpriority
		.word	_sys_setpriority
		.word	_sys_profil
		.word	_sys_statfs
		.word	_sys_fstatfs		/* 100 */
		.word	_sys_ioperm
		.word	_sys_socketcall
		.word	_sys_syslog
		.word	_sys_setitimer
		.word	_sys_getitimer		/* 105 */
		.word	_sys_newstat
		.word	_sys_newlstat
		.word	_sys_newfstat
		.word	_sys_uname
		.word	_sys_iopl		/* 110 */
		.word	_sys_vhangup
		.word	_sys_idle
		.word	_sys_vm86
		.word	_sys_wait4
		.word	_sys_swapoff		/* 115 */
		.word	_sys_sysinfo
		.word	_sys_ipc
		.word	_sys_fsync
		.word	_sys_sigreturn
		.word	_sys_clone		/* 120 */
		.word	_sys_setdomainname
		.word	_sys_newuname
		.word	_sys_modify_ldt
		.word	_sys_adjtimex
		.word	_sys_mprotect		/* 125 */
		.word	_sys_sigprocmask
		.word	_sys_create_module
		.word	_sys_init_module
		.word	_sys_delete_module
		.word	_sys_get_kernel_syms	/* 130 */
		.word	_sys_quotactl
		.word	_sys_getpgid
		.word	_sys_fchdir
		.word	_sys_bdflush
		.word	_sys_sysfs		/* 135 */
		.word	_sys_personality
		.word	0			/* for afs_syscall */
		.word	_sys_setfsuid
		.word	_sys_setfsgid
		.word	_sys_llseek		/* 140 */
		.space	(NR_syscalls-140)*4

spurious_text:	.asciz	"Spurious Count\n"
tlbs_text:	.asciz	"tlbs: %x\n"
mod_text:	.asciz	"mod: %x\n"
ab_text:	.asciz	"ab: %x\n"
ovfp_text:	.asciz	"ovfp: %x\n"
cpu_text:	.asciz	"cpu: %x\n"
sys_text:	.asciz	"syscall\n"
int_text:	.asciz	"int:\n"
tlbl2_text:	.asciz	"tlbl2: Table is at: %x\n"
tlbl3_text:	.asciz	"tlbl3: BadVAddr: %x Context: %x\n"
#if 0
hello_text:	.asciz	"hello, world\n"
hell_text:	.asciz	"Received int: %d %x\n"
#endif
