/*
 * linux/include/asm-mips/ptrace.h
 *
 * machine dependend structs and defines to help the user use
 * the ptrace system call.
 */
#ifndef _ASM_MIPS_PTRACE_H
#define _ASM_MIPS_PTRACE_H

/*
 * use ptrace (3 or 6, pid, PT_EXCL, data); to read or write
 * the processes registers.
 *
 * This defines/structures corrospond to the register layout on stack -
 * if the order here is changed, it needs to be updated in
 * arch/mips/fork.c:copy_process, asm/mips/signal.c:do_signal,
 * asm-mips/ptrace.c, include/asm-mips/ptrace.h.
 */

/*
 * Five words in the stackframe are reserved for the MIPS calling sequence
 */
#define IN_REG1		1
#define IN_REG2		((IN_REG1) + 1)
#define IN_REG3		((IN_REG2) + 1)
#define IN_REG4		((IN_REG3) + 1)
#define IN_REG5		((IN_REG4) + 1)
#define IN_REG6		((IN_REG5) + 1)
#define IN_REG7		((IN_REG6) + 1)
#define IN_REG8		((IN_REG7) + 1)
#define IN_REG9		((IN_REG8) + 1)
#define IN_REG10	((IN_REG9) + 1)
#define IN_REG11	((IN_REG10) + 1)
#define IN_REG12	((IN_REG11) + 1)
#define IN_REG13	((IN_REG12) + 1)
#define IN_REG14	((IN_REG13) + 1)
#define IN_REG15	((IN_REG14) + 1)
#define IN_REG16	((IN_REG15) + 1)
#define IN_REG17	((IN_REG16) + 1)
#define IN_REG18	((IN_REG17) + 1)
#define IN_REG19	((IN_REG18) + 1)
#define IN_REG20	((IN_REG19) + 1)
#define IN_REG21	((IN_REG20) + 1)
#define IN_REG22	((IN_REG21) + 1)
#define IN_REG23	((IN_REG22) + 1)
#define IN_REG24	((IN_REG23) + 1)
#define IN_REG25	((IN_REG24) + 1)

/*
 * k0 and k1 not saved
 */
#define IN_REG28	((IN_REG25) + 1)
#define IN_REG29	((IN_REG28) + 1)
#define IN_REG30	((IN_REG29) + 1)
#define IN_REG31	((IN_REG30) + 1)

/*
 * Saved special registers
 */
#define IN_LO		((IN_REG31) + 1)
#define IN_HI		((IN_LO) + 1)

/*
 * Saved cp0 registers
 */
#define IN_CP0_STATUS	((IN_HI) + 1)
#define IN_CP0_EPC	((IN_CP0_STATUS) + 1)
#define IN_CP0_ERROREPC	((IN_CP0_EPC) + 1)

/*
 * Some goodies...
 */
#define IN_INTERRUPT	((IN_CP0_ERROREPC) + 1)
#define IN_ORIG_REG2	((IN_INTERRUPT) + 1)

/*
 * this struct defines the way the registers are stored on the 
 * stack during a system call/exception. As usual the registers
 * k0/k1 aren't being saved.
 */

struct pt_regs {
	/*
	 * saved main processor registers
	 */
	long	        reg1,  reg2,  reg3,  reg4,  reg5,  reg6,  reg7;
	long	 reg8,  reg9, reg10, reg11, reg12, reg13, reg14, reg15;
	long	reg16, reg17, reg18, reg19, reg20, reg21, reg22, reg23;
	long	reg24, reg25,               reg28, reg29, reg30, reg31;
	/*
	 * Saved special registers
	 */
	long	lo;
	long	hi;
	/*
	 * saved cp0 registers
	 */
	unsigned long cp0_status;
	unsigned long cp0_epc;
	unsigned long cp0_errorepc;
	/*
	 * Some goodies...
	 */
	unsigned long interrupt;
	long orig_reg2;
};

/*
 * This function computes the interrupt number from the stack frame
 */
#define pt_regs2irq(p) ((int) ((struct pt_regs *)p)->interrupt)        

#endif /* _ASM_MIPS_PTRACE_H */
